@@ -1,7 +1,9 @@
 // This may look like C code, but it is really -*- C++ -*-
 //
 // Copyright Bob Friesenhahn, 1999, 2000, 2001, 2002, 2003
-// Copyright Dirk Lemstra 2014-2018
+//
+// Copyright @ 2014 ImageMagick Studio LLC, a non-profit organization
+// dedicated to making software imaging solutions freely available.
 //
 // Implementation of Options
 //
@@ -227,7 +229,7 @@ void Magick::Options::fileName(const std::string &fileName_)
     max_length;
 
   max_length=sizeof(_imageInfo->filename)-1;
-  fileName_.copy(_imageInfo->filename,max_length);
+  fileName_.copy(_imageInfo->filename,(size_t) max_length);
   if ((ssize_t) fileName_.length() > max_length)
     _imageInfo->filename[max_length]=0;
   else
@@ -470,9 +472,7 @@ Magick::ColorspaceType Magick::Options::quantizeColorSpace(void) const
 
 void Magick::Options::quantizeDither(const bool ditherFlag_)
 {
-  _imageInfo->dither=(MagickBooleanType) ditherFlag_;
-  _quantizeInfo->dither_method=ditherFlag_ ? RiemersmaDitherMethod :
-    NoDitherMethod;
+  quantizeDither(ditherFlag_ ? RiemersmaDitherMethod : NoDitherMethod);
 }
 
 bool Magick::Options::quantizeDither(void) const
@@ -480,6 +480,12 @@ bool Magick::Options::quantizeDither(void) const
   return(static_cast<bool>(_imageInfo->dither));
 }
 
+void Magick::Options::quantizeDither(const DitherMethod ditherMethod_)
+{
+  _imageInfo->dither=(MagickBooleanType) (ditherMethod_ != NoDitherMethod);
+  _quantizeInfo->dither_method=ditherMethod_;
+}
+
 void Magick::Options::quantizeDitherMethod(const DitherMethod ditherMethod_)
 {
   _quantizeInfo->dither_method=ditherMethod_;
@@ -586,17 +592,19 @@ void Magick::Options::strokeDashArray(const double *strokeDashArray_)
     {
       size_t
         x;
-      // Count elements in dash array
-      for (x=0; strokeDashArray_[x]; x++) ;
-      // Allocate elements
+
+      for (x=0; fabs(strokeDashArray_[x]) >= MagickEpsilon; x++) ;
       _drawInfo->dash_pattern=static_cast<double*>(AcquireMagickMemory((x+1)*
         sizeof(double)));
       if (!_drawInfo->dash_pattern)
         throwExceptionExplicit(MagickCore::ResourceLimitError,
           "Unable to allocate dash-pattern memory");
-      // Copy elements
-      memcpy(_drawInfo->dash_pattern,strokeDashArray_,(x+1)*sizeof(double));
-      _drawInfo->dash_pattern[x]=0.0;
+      else
+        {
+          (void) memset(_drawInfo->dash_pattern,0,(size_t) (x+1)*sizeof(double));
+          memcpy(_drawInfo->dash_pattern,strokeDashArray_,x*sizeof(double));
+          _drawInfo->dash_pattern[x]=0.0;
+        }
     }
 }
 
