@@ -17,7 +17,7 @@
 %                                October 2020                                 %
 %                                                                             %
 %                                                                             %
-%  Copyright @ 2020 ImageMagick Studio LLC, a non-profit organization         %
+%  Copyright @ 1999 ImageMagick Studio LLC, a non-profit organization         %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -171,10 +171,10 @@ static MagickBooleanType WriteKERNELImage(const ImageInfo *image_info,
   assert(image_info->signature == MagickCoreSignature);
   assert(image != (Image *) NULL);
   assert(image->signature == MagickCoreSignature);
-  if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickCoreSignature);
+  if (IsEventLogging() != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   status=OpenBlob(image_info,image,WriteBinaryBlobMode,exception);
   if (status == MagickFalse)
     return(status);
@@ -204,7 +204,7 @@ static MagickBooleanType WriteKERNELImage(const ImageInfo *image_info,
     {
       if ((x != 0) || (y != 0))
         (void) WriteBlobString(image,",");
-      if ((image->alpha_trait == BlendPixelTrait) &&
+      if (((image->alpha_trait != BlendPixelTrait) != 0) &&
           (GetPixelAlpha(image,p) < OpaqueAlpha/2))
         (void) WriteBlobString(image,"-");
       else
@@ -213,7 +213,7 @@ static MagickBooleanType WriteKERNELImage(const ImageInfo *image_info,
             GetMagickPrecision(),QuantumScale*GetPixelIntensity(image,p));
           (void) WriteBlobString(image,buffer);
         }
-      p+=GetPixelChannels(image);
+      p+=(ptrdiff_t) GetPixelChannels(image);
     }
     if (image->previous == (Image *) NULL)
       {
@@ -224,6 +224,7 @@ static MagickBooleanType WriteKERNELImage(const ImageInfo *image_info,
       }
   }
   (void) WriteBlobString(image,"\n");
-  (void) CloseBlob(image);
-  return(MagickTrue);
+  if (CloseBlob(image) == MagickFalse)
+    status=MagickFalse;
+  return(status);
 }
