@@ -78,6 +78,7 @@
 #include "MagickCore/token.h"
 #include "MagickCore/transform.h"
 #include "MagickCore/utility.h"
+#include "MagickCore/utility-private.h"
 #include "coders/bytebuffer-private.h"
 #include "coders/coders-private.h"
 #include "coders/ghostscript-private.h"
@@ -302,7 +303,7 @@ static void ReadPSInfo(const ImageInfo *image_info,Image *image,PSInfo *ps_info,
     {
       case '<':
       {
-        ReadGhostScriptXMPProfile(&buffer,&ps_info->xmp_profile);
+        ReadGhostScriptXMPProfile(&buffer,&ps_info->xmp_profile,exception);
         continue;
       }
       case '\n':
@@ -355,7 +356,7 @@ static void ReadPSInfo(const ImageInfo *image_info,Image *image,PSInfo *ps_info,
     if (CompareMagickByteBuffer(&buffer,ImageData,strlen(ImageData)) != MagickFalse)
       {
         p=GetMagickByteBufferDatum(&buffer);
-        (void) sscanf(p,ImageData " %lu %lu",&ps_info->columns,&ps_info->rows);
+        (void) MagickSscanf(p,ImageData " %lu %lu",&ps_info->columns,&ps_info->rows);
       }
     /*
       Is this a CMYK document?
@@ -439,18 +440,22 @@ static void ReadPSInfo(const ImageInfo *image_info,Image *image,PSInfo *ps_info,
         */
         if (SkipMagickByteBufferUntilNewline(&buffer) != MagickFalse)
           {
-            ps_info->icc_profile=AcquireStringInfo(MagickPathExtent);
-            datum=GetStringInfoDatum(ps_info->icc_profile);
-            for (i=0; (c=ProfileInteger(&buffer,hex_digits)) != EOF; i++)
-            {
-              if (i >= (ssize_t) GetStringInfoLength(ps_info->icc_profile))
+            ps_info->icc_profile=AcquireProfileStringInfo("icc",MagickPathExtent,
+              exception);
+            if (ps_info->icc_profile != (StringInfo*) NULL)
+              {
+                datum=GetStringInfoDatum(ps_info->icc_profile);
+                for (i=0; (c=ProfileInteger(&buffer,hex_digits)) != EOF; i++)
                 {
-                  SetStringInfoLength(ps_info->icc_profile,(size_t) i << 1);
-                  datum=GetStringInfoDatum(ps_info->icc_profile);
+                  if (i >= (ssize_t) GetStringInfoLength(ps_info->icc_profile))
+                    {
+                      SetStringInfoLength(ps_info->icc_profile,(size_t) i << 1);
+                      datum=GetStringInfoDatum(ps_info->icc_profile);
+                    }
+                  datum[i]=(unsigned char) c;
                 }
-              datum[i]=(unsigned char) c;
-            }
-            SetStringInfoLength(ps_info->icc_profile,(size_t) i+1);
+                SetStringInfoLength(ps_info->icc_profile,(size_t) i+1);
+              }
           }
         continue;
       }
@@ -468,7 +473,7 @@ static void ReadPSInfo(const ImageInfo *image_info,Image *image,PSInfo *ps_info,
         */
         p=GetMagickByteBufferDatum(&buffer);
         extent=0;
-        count=(ssize_t) sscanf(p,PhotoshopProfile " %lu",&extent);
+        count=(ssize_t) MagickSscanf(p,PhotoshopProfile " %lu",&extent);
         if ((count != 1) || (extent == 0))
           continue;
         if ((MagickSizeType) extent > GetBlobSize(image))
@@ -476,17 +481,21 @@ static void ReadPSInfo(const ImageInfo *image_info,Image *image,PSInfo *ps_info,
         length=(size_t) extent;
         if (SkipMagickByteBufferUntilNewline(&buffer) != MagickFalse)
           {
-            ps_info->photoshop_profile=AcquireStringInfo(length+1U);
-            q=GetStringInfoDatum(ps_info->photoshop_profile);
-            while (extent > 0)
-            {
-              c=ProfileInteger(&buffer,hex_digits);
-              if (c == EOF)
-                break;
-              *q++=(unsigned char) c;
-              extent-=MagickMin(extent,1);
-            }
-            SetStringInfoLength(ps_info->photoshop_profile,length);
+            ps_info->photoshop_profile=AcquireProfileStringInfo("8bim",
+              length+1U,exception);
+            if (ps_info->icc_profile != (StringInfo*) NULL)
+              {
+                q=GetStringInfoDatum(ps_info->photoshop_profile);
+                while (extent > 0)
+                {
+                  c=ProfileInteger(&buffer,hex_digits);
+                  if (c == EOF)
+                    break;
+                  *q++=(unsigned char) c;
+                  extent-=MagickMin(extent,1);
+                }
+                SetStringInfoLength(ps_info->photoshop_profile,length);
+              }
           }
         continue;
       }
@@ -500,35 +509,35 @@ static void ReadPSInfo(const ImageInfo *image_info,Image *image,PSInfo *ps_info,
     if (CompareMagickByteBuffer(&buffer,BoundingBox,strlen(BoundingBox)) != MagickFalse)
       {
         p=GetMagickByteBufferDatum(&buffer);
-        count=(ssize_t) sscanf(p,BoundingBox " %lf %lf %lf %lf",
+        count=(ssize_t) MagickSscanf(p,BoundingBox " %lf %lf %lf %lf",
           &bounds.x1,&bounds.y1,&bounds.x2,&bounds.y2);
         i=2;
       }
     if (CompareMagickByteBuffer(&buffer,DocumentMedia,strlen(DocumentMedia)) != MagickFalse)
       {
         p=GetMagickByteBufferDatum(&buffer);
-        count=(ssize_t) sscanf(p,DocumentMedia " %lf %lf %lf %lf",
+        count=(ssize_t) MagickSscanf(p,DocumentMedia " %lf %lf %lf %lf",
           &bounds.x1,&bounds.y1,&bounds.x2,&bounds.y2);
         i=1;
       }
     if (CompareMagickByteBuffer(&buffer,HiResBoundingBox,strlen(HiResBoundingBox)) != MagickFalse)
       {
         p=GetMagickByteBufferDatum(&buffer);
-        count=(ssize_t) sscanf(p,HiResBoundingBox " %lf %lf %lf %lf",
+        count=(ssize_t) MagickSscanf(p,HiResBoundingBox " %lf %lf %lf %lf",
           &bounds.x1,&bounds.y1,&bounds.x2,&bounds.y2);
         i=3;
       }
     if (CompareMagickByteBuffer(&buffer,PageBoundingBox,strlen(PageBoundingBox)) != MagickFalse)
       {
         p=GetMagickByteBufferDatum(&buffer);
-        count=(ssize_t) sscanf(p,PageBoundingBox " %lf %lf %lf %lf",
+        count=(ssize_t) MagickSscanf(p,PageBoundingBox " %lf %lf %lf %lf",
           &bounds.x1,&bounds.y1,&bounds.x2,&bounds.y2);
         i=1;
       }
     if (CompareMagickByteBuffer(&buffer,PageMedia,strlen(PageMedia)) != MagickFalse)
       {
         p=GetMagickByteBufferDatum(&buffer);
-        count=(ssize_t) sscanf(p,PageMedia " %lf %lf %lf %lf",
+        count=(ssize_t) MagickSscanf(p,PageMedia " %lf %lf %lf %lf",
           &bounds.x1,&bounds.y1,&bounds.x2,&bounds.y2);
         i=1;
       }
@@ -749,7 +758,7 @@ static Image *ReadPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
         strlen(translate_geometry));
     }
   (void) count;
-  file=close(file)-1;
+  file=close_utf8(file)-1;
   /*
     Render Postscript with the Ghostscript delegate.
   */
@@ -876,12 +885,11 @@ static Image *ReadPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
         }
     }
   if (info.icc_profile != (StringInfo *) NULL)
-    (void) SetImageProfile(image,"icc",info.icc_profile,exception);
+    (void) SetImageProfilePrivate(image,info.icc_profile,exception);
   if (info.photoshop_profile != (StringInfo *) NULL)
-    (void) SetImageProfile(image,"8bim",info.photoshop_profile,exception);
+    (void) SetImageProfilePrivate(image,info.photoshop_profile,exception);
   if (info.xmp_profile != (StringInfo *) NULL)
-    (void) SetImageProfile(image,"xmp",info.xmp_profile,exception);
-  CleanupPSInfo(&info);
+    (void) SetImageProfilePrivate(image,info.xmp_profile,exception);
   if (image_info->number_scenes != 0)
     {
       Image
@@ -1526,16 +1534,17 @@ static MagickBooleanType WritePSImage(const ImageInfo *image_info,Image *image,
     (void) ConcatenateMagickString(page_geometry,">",MagickPathExtent);
     (void) ParseMetaGeometry(page_geometry,&geometry.x,&geometry.y,
       &geometry.width,&geometry.height);
-    scale.x=PerceptibleReciprocal(resolution.x)*geometry.width*delta.x;
-    geometry.width=(size_t) floor(scale.x+0.5);
-    scale.y=PerceptibleReciprocal(resolution.y)*geometry.height*delta.y;
-    geometry.height=(size_t) floor(scale.y+0.5);
+    scale.x=MagickSafeReciprocal(resolution.x)*geometry.width*delta.x;
+    geometry.width=CastDoubleToSizeT(scale.x+0.5);
+    scale.y=MagickSafeReciprocal(resolution.y)*geometry.height*delta.y;
+    geometry.height=CastDoubleToSizeT(scale.y+0.5);
     (void) ParseAbsoluteGeometry(page_geometry,&media_info);
     (void) ParseGravityGeometry(image,page_geometry,&page_info,exception);
     if (image->gravity != UndefinedGravity)
       {
         geometry.x=(-page_info.x);
-        geometry.y=(ssize_t) (media_info.height+page_info.y-image->rows);
+        geometry.y=(ssize_t) media_info.height+page_info.y-(ssize_t)
+          image->rows;
       }
     pointsize=12.0;
     if (image_info->pointsize != 0.0)
@@ -1673,7 +1682,7 @@ static MagickBooleanType WritePSImage(const ImageInfo *image_info,Image *image,
                     if ((q-pixels+8) >= 80)
                       {
                         *q++='\n';
-                        (void) WriteBlob(image,q-pixels,pixels);
+                        (void) WriteBlob(image,(size_t) (q-pixels),pixels);
                         q=pixels;
                         (void) WriteBlobString(image,"%  ");
                       };
@@ -1688,7 +1697,7 @@ static MagickBooleanType WritePSImage(const ImageInfo *image_info,Image *image,
                   if ((q-pixels+8) >= 80)
                     {
                       *q++='\n';
-                      (void) WriteBlob(image,q-pixels,pixels);
+                      (void) WriteBlob(image,(size_t) (q-pixels),pixels);
                       q=pixels;
                       (void) WriteBlobString(image,"%  ");
                     };
@@ -1697,7 +1706,7 @@ static MagickBooleanType WritePSImage(const ImageInfo *image_info,Image *image,
             if (q != pixels)
               {
                 *q++='\n';
-                (void) WriteBlob(image,q-pixels,pixels);
+                (void) WriteBlob(image,(size_t) (q-pixels),pixels);
               }
             (void) WriteBlobString(image,"\n%%EndPreview\n");
             preview_image=DestroyImage(preview_image);
@@ -1741,9 +1750,9 @@ static MagickBooleanType WritePSImage(const ImageInfo *image_info,Image *image,
       bounds.x1=(double) geometry.x;
     if ((double) geometry.y < bounds.y1)
       bounds.y1=(double) geometry.y;
-    if ((double) (geometry.x+geometry.width-1) > bounds.x2)
+    if ((double) (geometry.x+(ssize_t) geometry.width-1) > bounds.x2)
       bounds.x2=(double) geometry.x+geometry.width-1;
-    if ((double) (geometry.y+(geometry.height+text_size)-1) > bounds.y2)
+    if ((double) (geometry.y+((ssize_t) geometry.height+(ssize_t) text_size)-1) > bounds.y2)
       bounds.y2=(double) geometry.y+(geometry.height+text_size)-1;
     value=GetImageProperty(image,"label",exception);
     if (value != (const char *) NULL)
@@ -1807,10 +1816,10 @@ static MagickBooleanType WritePSImage(const ImageInfo *image_info,Image *image,
                 if ((q-pixels+8) >= 80)
                   {
                     *q++='\n';
-                    (void) WriteBlob(image,q-pixels,pixels);
+                    (void) WriteBlob(image,(size_t) (q-pixels),pixels);
                     q=pixels;
                   }
-                p+=GetPixelChannels(image);
+                p+=(ptrdiff_t) GetPixelChannels(image);
               }
               if (image->previous == (Image *) NULL)
                 {
@@ -1823,7 +1832,7 @@ static MagickBooleanType WritePSImage(const ImageInfo *image_info,Image *image,
             if (q != pixels)
               {
                 *q++='\n';
-                (void) WriteBlob(image,q-pixels,pixels);
+                (void) WriteBlob(image,(size_t) (q-pixels),pixels);
               }
           }
         else
@@ -1859,13 +1868,13 @@ static MagickBooleanType WritePSImage(const ImageInfo *image_info,Image *image,
                     if ((q-pixels+2) >= 80)
                       {
                         *q++='\n';
-                        (void) WriteBlob(image,q-pixels,pixels);
+                        (void) WriteBlob(image,(size_t) (q-pixels),pixels);
                         q=pixels;
                       };
                     bit=0;
                     byte=0;
                   }
-                p+=GetPixelChannels(image);
+                p+=(ptrdiff_t) GetPixelChannels(image);
               }
               if (bit != 0)
                 {
@@ -1874,7 +1883,7 @@ static MagickBooleanType WritePSImage(const ImageInfo *image_info,Image *image,
                   if ((q-pixels+2) >= 80)
                     {
                       *q++='\n';
-                      (void) WriteBlob(image,q-pixels,pixels);
+                      (void) WriteBlob(image,(size_t) (q-pixels),pixels);
                       q=pixels;
                     }
                 };
@@ -1889,7 +1898,7 @@ static MagickBooleanType WritePSImage(const ImageInfo *image_info,Image *image,
             if (q != pixels)
               {
                 *q++='\n';
-                (void) WriteBlob(image,q-pixels,pixels);
+                (void) WriteBlob(image,(size_t) (q-pixels),pixels);
               }
           }
       }
@@ -1935,20 +1944,20 @@ static MagickBooleanType WritePSImage(const ImageInfo *image_info,Image *image,
                           if ((q-pixels+10) >= 80)
                             {
                               *q++='\n';
-                              (void) WriteBlob(image,q-pixels,pixels);
+                              (void) WriteBlob(image,(size_t) (q-pixels),pixels);
                               q=pixels;
                             }
                         }
                       length=0;
                     }
                   GetPixelInfoPixel(image,p,&pixel);
-                  p+=GetPixelChannels(image);
+                  p+=(ptrdiff_t) GetPixelChannels(image);
                 }
                 WriteRunlengthPacket(image,pixel,length,p);
                 if ((q-pixels+10) >= 80)
                   {
                     *q++='\n';
-                    (void) WriteBlob(image,q-pixels,pixels);
+                    (void) WriteBlob(image,(size_t) (q-pixels),pixels);
                     q=pixels;
                   }
                 if (image->previous == (Image *) NULL)
@@ -1962,7 +1971,7 @@ static MagickBooleanType WritePSImage(const ImageInfo *image_info,Image *image,
               if (q != pixels)
                 {
                   *q++='\n';
-                  (void) WriteBlob(image,q-pixels,pixels);
+                  (void) WriteBlob(image,(size_t) (q-pixels),pixels);
                 }
               break;
             }
@@ -1999,10 +2008,10 @@ static MagickBooleanType WritePSImage(const ImageInfo *image_info,Image *image,
                   if ((q-pixels+6) >= 80)
                     {
                       *q++='\n';
-                      (void) WriteBlob(image,q-pixels,pixels);
+                      (void) WriteBlob(image,(size_t) (q-pixels),pixels);
                       q=pixels;
                     }
-                  p+=GetPixelChannels(image);
+                  p+=(ptrdiff_t) GetPixelChannels(image);
                 }
                 if (image->previous == (Image *) NULL)
                   {
@@ -2015,7 +2024,7 @@ static MagickBooleanType WritePSImage(const ImageInfo *image_info,Image *image,
               if (q != pixels)
                 {
                   *q++='\n';
-                  (void) WriteBlob(image,q-pixels,pixels);
+                  (void) WriteBlob(image,(size_t) (q-pixels),pixels);
                 }
               break;
             }
@@ -2077,7 +2086,7 @@ static MagickBooleanType WritePSImage(const ImageInfo *image_info,Image *image,
                           if ((q-pixels+6) >= 80)
                             {
                               *q++='\n';
-                              (void) WriteBlob(image,q-pixels,pixels);
+                              (void) WriteBlob(image,(size_t) (q-pixels),pixels);
                               q=pixels;
                             }
                         }
@@ -2088,14 +2097,14 @@ static MagickBooleanType WritePSImage(const ImageInfo *image_info,Image *image,
                   pixel.green=(MagickRealType) GetPixelGreen(image,p);
                   pixel.blue=(MagickRealType) GetPixelBlue(image,p);
                   pixel.alpha=(MagickRealType) GetPixelAlpha(image,p);
-                  p+=GetPixelChannels(image);
+                  p+=(ptrdiff_t) GetPixelChannels(image);
                 }
                 q=PopHexPixel(hex_digits,(size_t) index,q);
                 q=PopHexPixel(hex_digits,(size_t) MagickMin(length,0xff),q);
                 if ((q-pixels+6) >= 80)
                   {
                     *q++='\n';
-                    (void) WriteBlob(image,q-pixels,pixels);
+                    (void) WriteBlob(image,(size_t) (q-pixels),pixels);
                     q=pixels;
                   }
                 if (image->previous == (Image *) NULL)
@@ -2109,7 +2118,7 @@ static MagickBooleanType WritePSImage(const ImageInfo *image_info,Image *image,
               if (q != pixels)
                 {
                   *q++='\n';
-                  (void) WriteBlob(image,q-pixels,pixels);
+                  (void) WriteBlob(image,(size_t) (q-pixels),pixels);
                 }
               break;
             }
@@ -2131,10 +2140,10 @@ static MagickBooleanType WritePSImage(const ImageInfo *image_info,Image *image,
                   if ((q-pixels+4) >= 80)
                     {
                       *q++='\n';
-                      (void) WriteBlob(image,q-pixels,pixels);
+                      (void) WriteBlob(image,(size_t) (q-pixels),pixels);
                       q=pixels;
                     }
-                  p+=GetPixelChannels(image);
+                  p+=(ptrdiff_t) GetPixelChannels(image);
                 }
                 if (image->previous == (Image *) NULL)
                   {
@@ -2147,7 +2156,7 @@ static MagickBooleanType WritePSImage(const ImageInfo *image_info,Image *image,
               if (q != pixels)
                 {
                   *q++='\n';
-                  (void) WriteBlob(image,q-pixels,pixels);
+                  (void) WriteBlob(image,(size_t) (q-pixels),pixels);
                 }
               break;
             }
@@ -2177,6 +2186,7 @@ static MagickBooleanType WritePSImage(const ImageInfo *image_info,Image *image,
       (void) WriteBlobString(image,buffer);
     }
   (void) WriteBlobString(image,"%%EOF\n");
-  (void) CloseBlob(image);
-  return(MagickTrue);
+  if (CloseBlob(image) == MagickFalse)
+    status=MagickFalse;
+  return(status);
 }
